/* Copyright (c) 2018  Paulo Costa
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
   * Neither the name of the copyright holders nor the names of
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. */

#include <Arduino.h>
#include "ESP32_cnc.h"
//#include <util/atomic.h>

//#define BIT(x) (0x01 << (x))

int toggle, even, adec;

motor_t motors[NUM_MOTORS];
volatile int motor_pos[NUM_MOTORS];
int motor_pos_ref[NUM_MOTORS], motor_pos_error[NUM_MOTORS];
bool motors_pos_ctrl[NUM_MOTORS];

#define MOTOR_CTRL_ACEL 200

// create a hardware timer
hw_timer_t * timer = NULL;


// Attach interrupt routine
void IRAM_ATTR onTimer(void)
{
  //DEBUG_PORT |= BIT(DEBUG_PIN);
  int i;

  even++;
  if (even & 0x01) {
    // Zero the step bits for motors 0 - 2 (X, Y, Z)
	  digitalWrite(motors[0].pin_step, 0);
	  digitalWrite(motors[1].pin_step, 0);

    // Update the direction bits
    // There must be 200 ns between changing dir and the begining of the step pulse (A4988 datsheet)
    // Set the direction pin
    //for(i = 0; i < 3; i++){
    //  if (motors[i].speed > 0) port |= motors[i].pin_dir;
    //}
    digitalWrite(motors[0].pin_dir, (motors[0].speed > 0));
    digitalWrite(motors[1].pin_dir, (motors[1].speed > 0));


    // Acceleration effect on speed (for all motors)
    adec++;
    if (adec > 100) {
      adec = 0;
      for(i = 0; i < NUM_MOTORS; i++){

        // Speed must increase -> positive accceleration
        if (motors[i].req_speed > motors[i].speed) {
          motors[i].speed += motors[i].accel;
          // But must stop on the req_speed value
          if (motors[i].speed > motors[i].req_speed) {
            motors[i].speed = motors[i].req_speed;
          }

        // Speed must decrease -> negative accceleration
        } else if (motors[i].req_speed < motors[i].speed) {
          motors[i].speed -= motors[i].accel;
          // But must stop on the req_speed value
          if (motors[i].speed < motors[i].req_speed) {
            motors[i].speed = motors[i].req_speed;
          }
        }
      }
    }
    // End or interrupt signal
    //DEBUG_PORT &= ~BIT(DEBUG_PIN);
    return;
  }



  // Step if there is an underflow or a overflow
  // A step pulse will last 1/INT_RATE  (it must be > 1 us: A4988 datsheet)
  for(i = 0; i < NUM_MOTORS; i++){
    motors[i].sub_pos += motors[i].speed;
    if (motors[i].sub_pos > ONE_STEP) {
      motors[i].sub_pos -= ONE_STEP;
      motor_pos[i]++;
      //port |= motors[i].pin_step;
      digitalWrite(motors[i].pin_step, 1);
    } else if (motors[i].sub_pos < 0) {
      motors[i].sub_pos += ONE_STEP;
      motor_pos[i]--;
      //port |= motors[i].pin_step;
      digitalWrite(motors[i].pin_step, 1);
    }
  }

  //DEBUG_PORT &= ~BIT(DEBUG_PIN);
  //digitalWrite(26, 1);

}

void set_motor_pos(byte mot, int new_pos)
{
  if (mot >= NUM_MOTORS) return;
  motor_pos[mot] = new_pos;
}

int get_motor_pos(byte mot)
{
  return motor_pos[mot];
}



// Position control
void set_motors_pos_ref(byte mot, int new_pos_ref)
{
  if (mot >= NUM_MOTORS) return;
  motor_pos_ref[mot] = new_pos_ref;
}

void set_motors_ctrl_state(byte mot, bool state)
{
  if (mot >= NUM_MOTORS) return;
  motors_pos_ctrl[mot] = state;
}

void update_motors_pos_ctrl()
{
  int i;
  for(i = 0; i < NUM_MOTORS; i++){
    if (motors_pos_ctrl[i]) {
      // Compute error
      motor_pos_error[i] = motor_pos_ref[i] - motor_pos[i];
      // Set speed
      set_motor_accel(i, MOTOR_CTRL_ACEL);
      set_motor_req_speed(i, motor_pos_error[i]);
    }
  }
}



void set_motor_speed(byte mot, int new_speed)
{
  if (mot >= NUM_MOTORS) return;
  motors[mot].speed = new_speed;
}

int get_motor_speed(byte mot)
{
  return motors[mot].speed;
}


void set_motor_req_speed(byte mot, int new_speed)
{
  if (mot >= NUM_MOTORS) return;
  motors[mot].req_speed = new_speed;
}

int get_motor_req_speed(byte mot)
{
  if (mot >= NUM_MOTORS) return 0;
  return motors[mot].req_speed;
}


void set_motor_accel(byte mot, byte new_accel)
{
  if (mot >= NUM_MOTORS) return;
  motors[mot].accel = new_accel;
}


void ESP32CNC_init(void)
{
  byte i;

  // Configure the steper drive pins as outputs
  pinMode(EN, OUTPUT);
  pinMode(X_DIR, OUTPUT);
  pinMode(X_STEP, OUTPUT);
  pinMode(Y_DIR, OUTPUT);
  pinMode(Y_STEP, OUTPUT);
  pinMode(Z_DIR, OUTPUT);
  pinMode(Z_STEP, OUTPUT);
  //pinMode(A_DIR, OUTPUT);
  //pinMode(A_STEP, OUTPUT);

  //pinMode(SPIN_EN, OUTPUT);

  // Configure the control pins as inputs with pullups
  //pinMode(X_ENDSTOP, INPUT_PULLUP);

  //pinMode(MOTOR_POWER_ON, INPUT_PULLUP);

  //pinMode(Y_ENDSTOP, INPUT_PULLUP);
  //digitalWrite(POWER_ON, 0); // Turn Power On
  //pinMode(POWER_ON, OUTPUT);

  //pinMode(Z_ENDSTOP, INPUT_PULLUP);

  //pinMode(ABORT, INPUT_PULLUP);
  //pinMode(BAT_CURRENT, INPUT);

  //pinMode(HOLD, INPUT_PULLUP);
  //pinMode(BAT_VOLTAGE, INPUT);


  for (i = 0; i < NUM_MOTORS; i++) {
    motors[i].speed = 0;// * ONE_STEP / INT_RATE;
    motors[i].req_speed = 0;//50 * i + 20;
    motors[i].accel = 50;
    motors_pos_ctrl[i] = false;
  }

  //motors[0].req_speed = -4000;
  //motors[1].req_speed = 4000;


  motors[0].pin_step = X_STEP;
  motors[1].pin_step = Y_STEP;
  //motors[2].pin_step = Z_STEP;

  motors[0].pin_dir = X_DIR;
  motors[1].pin_dir = Y_DIR;
  //motors[2].pin_dir = Z_DIR;

  // Use 1st timer of 4
  // 1 tick take 1/(80MHZ/80) = 1us so we set divider 80 and count up
  timer = timerBegin(0, 80, true);

  // Attach onTimer function to our timer
  timerAttachInterrupt(timer, &onTimer, true);

  // Set alarm to call onTimer function every second 1 tick is 1us  => 1 second is 1000000us
  // Repeat the alarm (third parameter)
  timerAlarmWrite(timer, 100, true);

  // Start an alarm
  timerAlarmEnable(timer);

  Serial.println("start timer");

  // Enable the X, Y, Z & Aux stepper outputs
  digitalWrite(EN, 0); //Low to enable

}
