/* Copyright (c) 2018  Paulo Costa
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
   * Neither the name of the copyright holders nor the names of
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. */

//#pragma GCC optimize ("-O3")

#ifndef ESP32_CNC_H
#define ESP32_CNC_H

#include <Arduino.h>

#define EN 13      // Enable pin for all stepper outputs

#define X_DIR 16   // PD5 - Direction pin for X axis
#define X_STEP 26  // PD2 - Step pin for X axis

#define Y_DIR 27   // PD6 - Direction pin for Y axis
#define Y_STEP 25  // PD3 - Step pin for Y axis

#define Z_DIR 14   // PD7 - Direction pin for Z axis
#define Z_STEP 17  // PD4 - Step pin for Z axis

//#define A_DIR 13  // PB5 - Direction pin for Aux driver. Requires D13 and A-DIR pins to be shorted
//#define A_STEP 12 // PB4 - Direction pin for Aux driver. Requires D12 and A-STEP pins to be shorted

//#define SPIN_DIR 13 // PB5 - Already used -> A_DIR
//#define SPIN_EN  12 // PB4 - Already used -> A_STEP

//#define X_ENDSTOP 9   // PB1 - X axis endstop input pin -> Debug Pin
//#define Y_ENDSTOP 10  // PB2 - Y axis endstop input pin -> Laser Control
//#define Z_ENDSTOP 11  // PB3 - Z axis endstop input pin
//#define ABORT A0  // PC0 - Abort input pin -> Battery current
//#define HOLD A1   // PC1 - Hold input pin -> Battery voltage
//#define RESUME A2 // PC2 Resume input pin



//#define DEBUG_OUT  X_ENDSTOP // PB1 - Debug pin (uses X_ENDSTOP)
//#define DEBUG_PIN  1 // PB1 - Debug pin (uses X_ENDSTOP)
//#define DEBUG_PORT PORTB // PB1 - Debug pin (uses X_ENDSTOP)

//#define POWER_ON        Y_ENDSTOP // PB1
//#define MOTOR_POWER_ON  Z_ENDSTOP // PB3

//#define BAT_CURRENT ABORT // PC0
//#define BAT_VOLTAGE HOLD   // PC1

#define ONE_STEP 10000
#define NUM_MOTORS 2
#define INT_RATE 10000

typedef struct{
    int sub_pos, speed, req_speed;
    byte accel;
    byte pin_step, pin_dir;
} motor_t;

void ESP32CNC_init(void);
void set_motor_pos(byte mot, int new_pos);
int  get_motor_pos(byte mot);
void set_motors_pos_ref(byte mot, int new_pos_ref);
void set_motors_ctrl_state(byte mot, bool state);
void update_motors_pos_ctrl();
void set_motor_speed(byte mot, int new_speed);
void set_motor_req_speed(byte mot, int new_speed);
int get_motor_req_speed(byte mot);
void set_motor_accel(byte mot, byte new_accel);

int get_motor_speed(byte mot);


#endif // ESP32_CNC_H
